from Utilities import *
from scipy.integrate import odeint
from matplotlib import rc

class ResProd:
    def __init__( self, Q0, C0):
        self.Resource0 = Q0
        self.Resource = Q0
        self.Capital = C0
        self.h_Extraction = Sigmoid( 0.5, 20.0, 0.0, 0.97)
        #self.h_Extraction.Plot( 0.0, 1.0)
        self.d_Depreciation = Sigmoid( 1980.0, 0.15, 0.04, 0.10)
        self.i_Reinvestment = Sigmoid( 1970.0, 0.10, 0.065, 0.101)
        self.c_Productivity = Sigmoid( 2000.0, 0.03, 1.00, 1.5)
        self.q_Efficiency = Sigmoid( 1970.0, 0.02, 1.00, 0.5)
        return
    def Z_Production( self, t):
        return self.c_Productivity.Compute( t) * self.Capital
    def W_Production( self, t):
        self._production = self.Z_Production( t)
        self._production *= self.h_Extraction.Compute( self.Resource/self.Resource0)
        return self._production
    def U_Production( self, t):
        U = self._production
        U *= (1.0 - self.i_Reinvestment.Compute( t))
        return U
    def dQ_dt( self, t):
        A = self._production
        A *= self.q_Efficiency.Compute( t)
        return -A
    def dC_dt( self, t):
        A = self._production
        A *= self.i_Reinvestment.Compute( t)
        A -= self.d_Depreciation.Compute( t) * self.Capital
        return A

RP = ResProd( 1230e3, 50)

def f( y, t):
    RP.Resource = y[0]
    RP.Capital = y[1]
    if RP.Resource <= 0.0:
        f0 = 0.0
        f1 = -RP.d_Depreciation( t) * RP.Capital
    else:
        RP.W_Production(t)
        f0 = RP.dQ_dt( t)
        f1 = RP.dC_dt( t)
    return [f0,f1]

# initial conditions
y0 = [RP.Resource, RP.Capital]          # initial condition vector
t  = np.linspace(1800, 2200, 1601)      # time grid

# solve the DEs
soln = odeint(f, y0, t)
Res = soln[:, 0]
Cap = soln[:, 1]

w = []
u = []
Rate = []
d = []
i = []
c = []
q = []
for j in range( len(t)):
    RP.Resource = Res[j]
    RP.Capital = Cap[j]
    w += [RP.W_Production( t[j])]
    u += [RP.U_Production( t[j])]
    Rate += [-RP.dQ_dt( t[j])]
    d += [RP.d_Depreciation.Compute( t[j]) * 100000.0]
    i += [RP.i_Reinvestment.Compute( t[j]) * 100000.0]
    c += [RP.c_Productivity.Compute( t[j]) * 10000.0]
    q += [RP.q_Efficiency.Compute( t[j]) * 10000.0]

font = {'family': 'Verdana',
        'weight': 'normal'}
rc('font', **font)
fig = plt.figure( figsize=(15,10))
Q = Res * 1e-2
plt.plot( t, Q, "ro", lw=3, label="Остаточные запасы x 1e-2")
plt.plot( t, w, "-", lw=3, label="Производство: всего")
plt.plot( t, u, "-", lw=3, label="Производство: средств потребления")
plt.plot( t, Rate, "-", lw=2, label="Добыча ресурсов")
plt.plot( t, d, "--", lw=2, label="% износа за год х 1е3")
plt.plot( t, i, "--", lw=2, label="% инвестиций х 1е3")
plt.plot( t, c, "--", lw=2, label="производительность капитала х 1е4")
plt.plot( t, q, "--", lw=2, label="затратность производства х 1е4")
tcal, vcal = Load_Calibration( "Resources_Calibration.csv", "Year", "Total")
plt.plot( tcal, vcal, "r+", lw=3, ms=7, mew=2, label="Реальные Нефть+Уголь+Газ")
plt.xlabel("Годы")
plt.xlim( 1900, 2100)
plt.ylabel("миллионов тонн условной продукции")
plt.title( "Калибровка: По совокупности производства углеводородного сырья")
plt.grid(True)
plt.legend(loc=0)
plt.savefig( ".\\0A_Testing\\figure_9.png")
fig.show()
#fig.clear()
#plt.clf()
